/*
 *  sicocon.h
 *  PhatBoy Control
 *
 *  Created by Arne Scheffler on Sat Feb 22 2003.
 *  Copyright (c) 2003 Arne Scheffler. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __sicocon__
#define __sicocon__

#ifndef __midieffect__
#include "midieffect.h"
#endif

#ifndef __linkedlist__
#include "linkedlist.h"
#endif

#ifndef __plugparams__
#include "plugparams.h"
#endif

//------------------------------------------------------------------------
class PhatBoyMap
{
public:
	PhatBoyMap ()
	: mapcc99 (false)
	, mapcc98 (false)
	, mapcc6 (false)
	{
		for (char m = 0; m < 3; m++)
		{
			for (char i = 0; i < 13; i++)
				ccOut[m][i] = -1;
		}
	}

	tresult processMidi (IMEObjectID event, IMEAccessor* accessor)
	{
		if (processMap3 (event, accessor) == kResultTrue)
			return kResultTrue;
//		if (processMap1 (event, accessor) == kResultTrue)
//			return kResultTrue;
//		if (processMap2 (event, accessor) == kResultTrue)
//			return kResultTrue;
		return kResultFalse;
	}
	
	void setOutValue (char map, char index, int value) { if (index < 13 && map < 3) ccOut[map][index] = value; }
	int  getOutValue (char map, char index) { return (index < 13 && map < 3) ? ccOut[map][index] : 0; }

//------------------------------------------------------------------------
protected:
	int ccOut[3][13];
	char mapIndex;
	bool mapcc99, mapcc98, mapcc6;
	int mapcc, ccdata;
	
	tresult processMap1 (IMEObjectID event, IMEAccessor* accessor);
	tresult processMap2 (IMEObjectID event, IMEAccessor* accessor);
	tresult processMap3 (IMEObjectID event, IMEAccessor* accessor);
};

//------------------------------------------------------------------------
class SiCoCon: public CMidiEffect
{
public:
//------------------------------------------------------------------------
	SiCoCon (const char* uid);
	~SiCoCon ();

	DELEGATE_BASIC_PLUGIN_METHODS

	static FUnknown* createInstance (void* context)
	{
		return (IMidiEffect*)new SiCoCon ((char*)context);
	}

	virtual tresult PLUGIN_API parameterChanged (IParameter*, long tag);	

	// CVSTMidiProcessor overrides
	virtual tresult initializePlugin (FUnknown* context);
	virtual tresult terminatePlugin ();
	virtual tresult PLUGIN_API receiveEvent (IMEObjectID event);
	virtual tresult PLUGIN_API playAction (long fromPPQ, long toPPQ, bool immediate);
	virtual tresult PLUGIN_API positAction (long position);
	virtual tresult PLUGIN_API swapAction (long position, long length);
	virtual tresult PLUGIN_API stopAction (long position);
	virtual tresult PLUGIN_API startAction (long position);

	virtual tresult PLUGIN_API setChunk (char* chunk, long size);
	virtual tresult PLUGIN_API getChunk (char* chunk, long* size);

	enum {
		kPBCCTag,
		kCCOutTag,
		kNumParameters
	};
//------------------------------------------------------------------------
protected:
	long playPos;
	bool learn;
	PhatBoyMap map;

	CParameter* paramPBCC;
	CParameter* paramCCOut;
	
	void makeParameters ();
};

#endif

